/****************************TEMPERATURE***************************/

// Read out temperature from ADC and build average if requested.

void read_temperature(uint8_t build_avg)
{
    uint32_t temp_calc = 0;
    uint8_t i,j,filtered_temp_values = 0;
    uint16_t adc_old = 0;
    
    for (i=0;i<ADC_READINGS;i++)
        {
        ADCSRA |= (1<<ADSC);
        //read out adc
        while (ADCSRA & (1<<ADSC))
            {
            }
        if ((ADC+SENSOROFFSET) > SENSORNULL && ADC <= SENSOR100)
            {
            //we have a valid ADC reading so lets filter out spikes and strong noise by comparing it with last ADC value.
            if (adc_old == 0 || ADC_FILTER_SHARPNESS == 0 || ((ADC > adc_old - ADC_FILTER_SHARPNESS) && (ADC < adc_old + ADC_FILTER_SHARPNESS)))
                {
                temp_calc+=ADC;
                adc_old = ADC;
                filtered_temp_values++;
                }
            }
        else
            {
            //Sensor reading is out of limits, so no sensor is connected or reading is unreliable.
            temperature = 0;
            rgb_temperature = 0;
            return;
            }
        }
    //average of ADC readings
    temp_calc = temp_calc / filtered_temp_values;
    
    //correction via temperature offset
    if (tempoffset <= 20 && temp_calc > 20)
        temp_calc = temp_calc - (20 - tempoffset);
    else
        temp_calc = temp_calc + (tempoffset - 20);
    
    //simple calculation for temp including correction factors.
    //1000 100.0 degress spreading without "." 
    temp_calc = ((temp_calc+SENSOROFFSET)-SENSORNULL)*1000/(SENSOR100-SENSORNULL);
    
    //build average over multiple reading cycles if requested, otherwise write temp_calc directly into temperature variable for displaying it.
    if (build_avg == TRUE && TEMP_AVG_DEEP != 0)
        {
        //floating average, but too big for atmega168 ;(
        //for (j=avg_count;j>0;j--)
        //    temp_avg[j] = temp_avg[j-1];
        //temp_avg[0] = temp_calc;
        temp_avg[avg_count] = temp_calc;
        avg_count++;
        if (avg_count == TEMP_AVG_DEEP)
            {
            temperature = 0;
            for (j=0;j<avg_count;j++)
                temperature+=temp_avg[j];
            temperature = temperature / avg_count;
            avg_count = 0;
            }
        }
    else
        temperature = temp_calc;
    rgb_temperature = temperature / 10;
}



